#ifndef _VECTOR_H_
#define _VECTOR_H_
#include <math.h>

class Vector {
  public:
    float x, y, z;

    Vector() {};
    Vector(float v) { x = y = x = v; };
    Vector(const Vector& v) { x = v.x; y = v.y; z = v.z; };
    Vector(float vx, float vy, float vz) { x = vx; y = vy; z = vz; };

    Vector& operator = (const Vector& v) { x = v.x; y = v.y; z = v.z; return *this; }
    Vector& operator = (float f) { x = y = z = f; return *this; };
    Vector operator - () const { return Vector(-x, -y, -z); };
    Vector& operator += (const Vector& v)
    {
      x += v.x;
      y += v.y;
      z += v.z;

      return *this;
    }
    Vector& operator -= (const Vector& v)
    {
      x -= v.x;
      y -= v.y;
      z -= v.z;

      return *this;
    }
    Vector& operator *= (const Vector& v)
    {
      x *= v.x;
      y *= v.y;
      z *= v.z;

      return *this;
    }
    Vector& operator *= (float v)
    {
      x *= v;
      y *= v;
      z *= v;

      return *this;
    }
    Vector& operator /= (float v)
    {
      x /= v;
      y /= v;
      z /= v;

      return *this;
    }

    friend Vector operator + (const Vector& u, const Vector& v)
    {
      return Vector(u.x + v.x, u.y + v.y, u.z + v.z);
    }
    friend Vector operator - (const Vector& u, const Vector& v)
    {
      return Vector(u.x - v.x, u.y - v.y, u.z - v.z);
    }
    friend Vector operator * (const Vector& u, const Vector& v)
    {
      return Vector(u.x * v.x, u.y * v.y, u.z * v.z);
    }
    friend Vector operator * (float f, const Vector& u)
    {
      return Vector(u.x * f, u.y * f, u.z * f);
    }
    friend Vector operator * (const Vector& u, float f)
    {
      return Vector(u.x * f, u.y * f, u.z * f);
    }
    friend Vector operator / (const Vector& u, const Vector& v)
    {
      return Vector(u.x / v.x, u.y / v.y, u.z / v.z);
    }
    friend Vector operator / (const Vector& u, float f)
    {
      return Vector(u.x / f, u.y / f, u.z / f);
    }
    friend float operator & (const Vector& u, const Vector& v)
    {
      return u.x * v.x + u.y * v.y + u.z * v.z;
    }
    friend Vector operator ^ (const Vector& u, const Vector& v)
    {
      return Vector(u.y * v.z - u.z * v.y,
                    u.z * v.x - u.x * v.z,
                    u.x * v.y - u.y * v.x);
    }

    float operator ! () { return (float)sqrt(x * x + y * y + z * z); };
    float& operator [] (int n) { return *(&x + n); };
    int operator < (float v) { return x < v && y < v && z < v; };
    int operator > (float v) { return x > v && y > v && z > v; };
};


inline Vector Normalize(Vector& v) { return v / !v; };

Vector& Clip(Vector& v)
{
  if (v.x < 0)
    v.x = 0;
  else if (v.x > 1)
    v.x = 1;

  if (v.y < 0)
    v.y = 0;
  else if (v.y > 1)
    v.y = 1;

  if (v.z < 0)
    v.z = 0;
  else if (v.z > 1)
    v.z = 1;

  return v;
}


#endif
